{% extends 'dcim/moduletype/base.html' %}
{% load buttons %}
{% load helpers %}
{% load plugins %}

{% block content %}
  <div class="row">
    <div class="col col-md-6">
      <div class="card">
        <h5 class="card-header">Module Type</h5>
        <div class="card-body">
          <table class="table table-hover attr-table">
            <tr>
              <td>Manufacturer</td>
              <td>{{ object.manufacturer|linkify }}</td>
            </tr>
            <tr>
              <td>Model Name</td>
              <td>{{ object.model }}</td>
            </tr>
            <tr>
              <td>Part Number</td>
              <td>{{ object.part_number|placeholder }}</td>
            </tr>
            <tr>
              <td>Instances</td>
              <td><a href="{% url 'dcim:module_list' %}?module_type_id={{ object.pk }}">{{ instance_count }}</a></td>
            </tr>
          </table>
        </div>
      </div>
      {% include 'inc/panels/custom_fields.html' %}
      {% plugin_left_page object %}
    </div>
    <div class="col col-md-6">
      {% include 'inc/panels/tags.html' %}
      {% include 'inc/panels/comments.html' %}
      {% include 'inc/panels/image_attachments.html' %}
      {% plugin_right_page object %}
    </div>
  </div>
  <div class="row">
    <div class="col col-md-12">
      {% plugin_full_width_page object %}
    </div>
  </div>
{% endblock %}
