<a href="#"
  hx-get="{{ url }}"
  hx-target="#htmx-modal-content"
  class="btn btn-sm btn-danger"
  data-bs-toggle="modal"
  data-bs-target="#htmx-modal"
>
  <span class="mdi mdi-trash-can-outline" aria-hidden="true"></span>&nbsp;Delete
</a>
