{% load helpers %}

<div class="card">
  <h5 class="card-header">Tags</h5>
  <div class="card-body">
    {% with url=object|validated_viewname:"list" %}
      {% for tag in object.tags.all %}
        {% tag tag url %}
      {% empty %}
        <span class="text-muted">No tags assigned</span>
      {% endfor %}
    {% endwith %}
  </div>
</div>
