{% extends 'generic/object.html' %}
{% load buttons %}
{% load helpers %}
{% load static %}
{% load plugins %}

{% block breadcrumbs %}
  {{ block.super }}
  <li class="breadcrumb-item"><a href="{% url 'dcim:rackreservation_list' %}?rack_id={{ object.rack.pk }}">{{ object.rack }}</a></li>
  <li class="breadcrumb-item">Units {{ object.unit_list }}</li>
{% endblock %}

{% block content %}
<div class="row mb-3">
	<div class="col col-12 col-xl-5">
        <div class="card">
            <h5 class="card-header">
                Rack
            </h5>
            <div class="card-body">
                <table class="table table-hover attr-table">
                    {% with rack=object.rack %}
                        <tr>
                            <th scope="row">Site</th>
                            <td>
                                {% if rack.site.region %}
                                    {{ rack.site.region|linkify }} /
                                {% endif %}
                                {{ rack.site|linkify }}
                            </td>
                        </tr>
                        <tr>
                            <th scope="row">Location</th>
                            <td>{{ rack.location|linkify|placeholder }}</td>
                        </tr>
                        <tr>
                            <th scope="row">Rack</th>
                            <td>{{ rack|linkify }}</td>
                        </tr>
                    {% endwith %}
                </table>
            </div>
        </div>
        <div class="card">
            <h5 class="card-header">
                Reservation Details
            </h5>
            <div class="card-body">
                <table class="table table-hover attr-table">
                    <tr>
                        <th scope="row">Units</th>
                        <td>{{ object.unit_list }}</td>
                    </tr>
                    <tr>
                        <th scope="row">Tenant</th>
                        <td>
                            {% if object.tenant.group %}
                                {{ object.tenant.group|linkify }} /
                            {% endif %}
                            {{ object.tenant|linkify|placeholder }}
                        </td>
                    </tr>
                    <tr>
                        <th scope="row">User</th>
                        <td>{{ object.user }}</td>
                    </tr>
                    <tr>
                        <th scope="row">Description</th>
                        <td>{{ object.description }}</td>
                    </tr>
                </table>
            </div>
        </div>
        {% include 'inc/panels/custom_fields.html' %}
        {% include 'inc/panels/tags.html' %}
        {% plugin_left_page object %}
	</div>
    <div class="col col-12 col-xl-7">
        <div class="row" style="margin-bottom: 20px">
            <div class="col col-md-6 col-sm-6 col-xs-12 text-center">
                <div style="margin-left: 30px">
                    <h4>Front</h4>
                    {% include 'dcim/inc/rack_elevation.html' with object=object.rack face='front' %}
                </div>
            </div>
            <div class="col col-md-6 col-sm-6 col-xs-12 text-center">
                <div style="margin-left: -30px">
                    <h4>Rear</h4>
                    {% include 'dcim/inc/rack_elevation.html' with object=object.rack face='rear' %}
                </div>
            </div>
        </div>
        {% plugin_right_page object %}
    </div>
</div>
<div class="row">
    <div class="col col-md-12">
        {% plugin_full_width_page object %}
    </div>
</div>
{% endblock %}
