{% if utilization == 0 %}
  <div class="progress align-items-center justify-content-center">
    <span class="w-100 text-center">{{ utilization }}%</span>
  </div>
{% else %}
  <div class="progress">
    <div
      role="progressbar"
      aria-valuemin="0"
      aria-valuemax="100"
      aria-valuenow="{{ utilization }}"
      class="progress-bar {{ bar_class }}"
      style="width: {{ utilization }}%;"
    >
      {% if utilization >= 25 %}{{ utilization|floatformat:0 }}%{% endif %}
    </div>
    {% if utilization < 25 %}
      <span class="ps-1">{{ utilization|floatformat:0 }}%</span>
    {% endif %}
  </div>
{% endif %}
