{% if applied_filters %}
  <div class="mb-3">
    {% for filter in applied_filters %}
      <a href="{{ filter.link_url }}" class="badge rounded-pill bg-primary text-decoration-none me-1">
        <i class="mdi mdi-close"></i> {{ filter.link_text }}
      </a>
    {% endfor %}
    {% if applied_filters|length > 1 %}
      <a href="?" class="badge rounded-pill bg-danger text-decoration-none me-1">
        <i class="mdi mdi-tag-off"></i> Clear all
      </a>
    {% endif %}
  </div>
{% endif %}
