{% extends 'users/base.html' %}
{% load form_helpers %}

{% block title %}Change Password{% endblock %}

{% block content %}
    <form action="." method="post" class="form form-horizontal col-md-8 offset-md-2">
        {% csrf_token %}
        <div class="field-group">
            <h5 class="text-center">Password</h5>
            {% render_field form.old_password %}
            {% render_field form.new_password1 %}
            {% render_field form.new_password2 %}
        </div>
        <div class="text-end">
            <a href="{% url 'user:profile' %}" class="btn btn-outline-danger">Cancel</a>
            <button type="submit" name="_update" class="btn btn-primary">Save</button>
        </div>
    </form>
{% endblock %}
