{% extends 'ipam/vlan/base.html' %}
{% load helpers %}

{% block content %}
  {% include 'inc/table_controls_htmx.html' with table_modal="VLANDevicesTable_config" %}
  
  <form method="post">
    {% csrf_token %}
    <div class="card">
      <div class="card-body" id="object_list">
        {% include 'htmx/table.html' %}
      </div>
    </div>
  </form>
{% endblock content %}

{% block modals %}
  {{ block.super }}
  {% table_config_form table %}
{% endblock modals %}
