{% extends 'generic/object_edit.html' %}
{% load form_helpers %}

{% block form %}
    <div class="card">
        <h5 class="card-header">Config Context</h5>
        <div class="card-body">
            {% render_field form.name %}
            {% render_field form.weight %}
            {% render_field form.description %}
            {% render_field form.is_active %}
        </div>
    </div>
    <div class="card">
        <h5 class="card-header">Assignment</h5>
        <div class="card-body">
            {% render_field form.regions %}
            {% render_field form.site_groups %}
            {% render_field form.sites %}
            {% render_field form.device_types %}
            {% render_field form.roles %}
            {% render_field form.platforms %}
            {% render_field form.cluster_types %}
            {% render_field form.cluster_groups %}
            {% render_field form.clusters %}
            {% render_field form.tenant_groups %}
            {% render_field form.tenants %}
            {% render_field form.tags %}
        </div>
    </div>
    <div class="card">
        <h5 class="card-header">Data</h5>
        <div class="card-body">
            {% render_field form.data %}
        </div>
    </div>
{% endblock %}
