# Generated by Django 3.2.8 on 2021-10-21 20:26

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('virtualization', '0025_extend_tag_support'),
    ]

    operations = [
        migrations.AddField(
            model_name='vminterface',
            name='bridge',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='bridge_interfaces', to='virtualization.vminterface'),
        ),
    ]
