{% extends 'generic/object_edit.html' %}
{% load helpers %}
{% load form_helpers %}

{% block form %}
  <div class="field-group mb-5">
    <div class="row mb-3">
      <label class="col-sm-3 col-form-label text-lg-end required">
        {{ object.parent|meta:"verbose_name"|bettertitle }}
      </label>
      <div class="col-sm-9">
        <div class="form-control-plaintext">
          {{ object.parent|linkify }}
        </div>
      </div>
    </div>
    {% render_form form %}
  </div>
{% endblock form %}
