{% load helpers %}
<table class="table table-hover panel-body attr-table">
    {% if termination.device %}
        {# Device component #}
        <tr>
            <td>Device</td>
            <td>{{ termination.device|linkify }}</td>
        </tr>
        <tr>
            <td>Site</td>
            <td>{{ termination.device.site|linkify }}</td>
        </tr>
        {% if termination.device.rack %}
            <tr>
                <td>Rack</td>
                <td>{{ termination.device.rack|linkify }}</td>
            </tr>
        {% endif %}
        <tr>
            <td>Type</td>
            <td>{{ termination|meta:"verbose_name"|capfirst }}</td>
        </tr>
        <tr>
            <td>Component</td>
            <td>{{ termination|linkify }}</td>
        </tr>
    {% else %}
        {# Circuit termination #}
        <tr>
            <td>Provider</td>
            <td>{{ termination.circuit.provider|linkify }}</td>
        </tr>
        <tr>
            <td>Circuit</td>
            <td>{{ termination.circuit|linkify }}</td>
        </tr>
        <tr>
            <td>Termination</td>
            <td>{{ termination }}</td>
        </tr>
    {% endif %}
</table>
