{% extends 'generic/object.html' %}
{% load buttons %}
{% load helpers %}
{% load perms %}
{% load plugins %}

{% block breadcrumbs %}
  {{ block.super }}
  {% if object.device %}
    <li class="breadcrumb-item"><a href="{% url 'ipam:service_list' %}?device_id={{ object.device.pk }}">{{ object.device }}</a></li>
  {% elif object.virtual_machine %}
    <li class="breadcrumb-item"><a href="{% url 'ipam:service_list' %}?virtual_machine_id={{ object.virtual_machine.pk }}">{{ object.virtual_machine }}</a></li>
  {% endif %}
{% endblock %}

{% block content %}
<div class="row mb-3">
	<div class="col col-md-6">
        <div class="card">
            <h5 class="card-header">
                Service
            </h5>
            <div class="card-body">
                <table class="table table-hover attr-table">
                    <tr>
                        <th scope="row">Name</th>
                        <td>{{ object.name }}</td>
                    </tr>
                    <tr>
                        <th scope="row">Parent</th>
                        <td>{{ object.parent|linkify }}</td>
                    </tr>
                    <tr>
                        <th scope="row">Protocol</th>
                        <td>{{ object.get_protocol_display }}</td>
                    </tr>
                    <tr>
                        <th scope="row">Ports</th>
                        <td>{{ object.port_list }}</td>
                    </tr>
                    <tr>
                        <th scope="row">IP Addresses</th>
                        <td>
                            {% for ipaddress in object.ipaddresses.all %}
                                {{ ipaddress|linkify }}<br />
                            {% empty %}
                                <span class="text-muted">None</span>
                            {% endfor %}
                        </td>
                    </tr>
                    <tr>
                        <th scope="row">Description</th>
                        <td>{{ object.description|placeholder }}</td>
                    </tr>
                </table>
            </div>
        </div>
        {% plugin_left_page object %}
    </div>
    <div class="col col-md-6">
        {% include 'inc/panels/custom_fields.html' %}
        {% include 'inc/panels/tags.html' %}
        {% plugin_right_page object %}
    </div>
</div>
<div class="row mb-3">
    <div class="col col-md-12">
        {% plugin_full_width_page object %}
    </div>
</div>
{% endblock %}
