<div class="modal fade" id="htmx-modal" tabindex="-1" aria-hidden="true">
  <div class="modal-dialog modal-dialog-centered">
    <div class="modal-content" id="htmx-modal-content">
      {# Dynamic content goes here #}
    </div>
  </div>
</div>
