{% load helpers %}

<div class="card">
  <h5 class="card-header">
    Comments
  </h5>
  <div class="card-body">
    {% if object.comments %}
      {{ object.comments|markdown }}
    {% else %}
      <span class="text-muted">None</span>
    {% endif %}
  </div>
</div>
