{% extends 'rest_framework/base.html' %}
{% load static %}

{% block head %}
  {{ block.super }}
  <link rel="icon" type="image/png" href="{% static 'rest-api.ico' %}" />
{% endblock head %}

{% block title %}{% if name %}{{ name }} | {% endif %}NetBox REST API{% endblock %}

{% block branding %}
  <a class="navbar-brand" href="{% url 'home' %}">NetBox</a>
{% endblock branding %}
