{% extends 'generic/object.html' %}
{% load helpers %}
{% load plugins %}

{% block breadcrumbs %}
  {{ block.super }}
  <li class="breadcrumb-item">
    <a href="{% url 'dcim:device_inventory' pk=object.device.pk %}">{{ object.device }}</a>
  </li>
{% endblock %}

{% block content %}
    <div class="row mb-3">
        <div class="col col-md-6">
            <div class="card">
                <h5 class="card-header">Inventory Item</h5>
                <div class="card-body">
                    <table class="table table-hover attr-table">
                        <tr>
                            <th scope="row">Device</th>
                            <td>{{ object.device|linkify }}</td>
                        </tr>
                        <tr>
                            <th scope="row">Parent Item</th>
                            <td>{{ object.parent|linkify|placeholder }}</td>
                        </tr>
                        <tr>
                            <th scope="row">Name</th>
                            <td>{{ object.name }}</td>
                        </tr>
                        <tr>
                            <th scope="row">Label</th>
                            <td>{{ object.label|placeholder }}</td>
                        </tr>
                        <tr>
                            <th scope="row">Role</th>
                            <td>{{ object.role|linkify|placeholder }}</td>
                        </tr>
                        <tr>
                            <th scope="row">Component</th>
                            <td>{{ object.component|linkify|placeholder }}</td>
                        </tr>
                        <tr>
                            <th scope="row">Manufacturer</th>
                            <td>{{ object.manufacturer|linkify|placeholder }}</td>
                        </tr>
                        <tr>
                            <th scope="row">Part ID</th>
                            <td>{{ object.part_id|placeholder }}</td>
                        </tr>
                        <tr>
                            <th scope="row">Serial</th>
                            <td>{{ object.serial|placeholder }}</td>
                        </tr>
                        <tr>
                            <th scope="row">Asset Tag</th>
                            <td>{{ object.asset_tag|placeholder }}</td>
                        </tr>
                        <tr>
                            <th scope="row">Description</th>
                            <td>{{ object.description|placeholder }}</td>
                        </tr>
                    </table>
                </div>
            </div>
            {% include 'inc/panels/custom_fields.html' %}
            {% include 'inc/panels/tags.html' %}
            {% plugin_left_page object %}
        </div>
        <div class="col col-md-6">
            {% plugin_right_page object %}
        </div>
    </div>
    <div class="row mb-3">
        <div class="col col-md-12">
            {% plugin_full_width_page object %}
        </div>
    </div>
{% endblock %}
