<td>
    {% if termination.parent_object.provider %}
        <i class="mdi mdi-lightning-bolt" title="Circuit"></i>
        <a href="{{ termination.parent_object.get_absolute_url }}">
            {{ termination.parent_object.provider }}
            {{ termination.parent_object }}
        </a>
    {% else %}
        {{ termination.parent_object|linkify }}
    {% endif %}
</td>
<td>
    {{ termination|linkify }}
</td>
