{% extends 'base/layout.html' %}
{% load helpers %}

{% block title %}Reports{% endblock %}

{% block tabs %}
  <ul class="nav nav-tabs px-3">
    <li class="nav-item" role="presentation">
      <a class="nav-link active" role="tab">Reports</a>
    </li>
  </ul>
{% endblock tabs %}

{% block content-wrapper %}
  <div class="tab-content">
    {% if reports %}
      {% for module, module_reports in reports %}
        <div class="card">
          <h5 class="card-header">
            <a name="module.{{ module }}"></a>
            <i class="mdi mdi-file-document-outline"></i> {{ module|bettertitle }}
          </h5>
          <div class="card-body">
            <table class="table table-hover table-headings reports">
              <thead>
                <tr>
                  <th width="250">Name</th>
                  <th width="110">Status</th>
                  <th>Description</th>
                  <th width="150" class="text-end">Last Run</th>
                  <th width="120"></th>
                </tr>
              </thead>
              <tbody>
                {% for report in module_reports %}
                  <tr>
                    <td>
                      <a href="{% url 'extras:report' module=report.module name=report.class_name %}" id="{{ report.module }}.{{ report.class_name }}">{{ report.name }}</a>
                    </td>
                    <td>
                      {% include 'extras/inc/job_label.html' with result=report.result %}
                    </td>
                    <td>{{ report.description|markdown|placeholder }}</td>
                    <td class="text-end">
                      {% if report.result %}
                        <a href="{% url 'extras:report_result' job_result_pk=report.result.pk %}">{{ report.result.created|annotated_date }}</a>
                      {% else %}
                        <span class="text-muted">Never</span>
                      {% endif %}
                    </td>
                    <td>
                      {% if perms.extras.run_report %}
                        <div class="float-end noprint">
                          <form action="{% url 'extras:report' module=report.module name=report.class_name %}" method="post">
                            {% csrf_token %}
                            <button type="submit" name="_run" class="btn btn-primary btn-sm" style="width: 110px">
                              {% if report.result %}
                                <i class="mdi mdi-replay"></i> Run Again
                              {% else %}
                                <i class="mdi mdi-play"></i> Run Report
                              {% endif %}
                            </button>
                          </form>
                        </div>
                      {% endif %}
                    </td>
                  </tr>
                  {% for method, stats in report.result.data.items %}
                    <tr>
                      <td colspan="4" class="method">
                        <span class="ps-3">{{ method }}</span>
                      </td>
                      <td class="text-end text-nowrap report-stats">
                        <span class="badge bg-success">{{ stats.success }}</span>
                        <span class="badge bg-info">{{ stats.info }}</span>
                        <span class="badge bg-warning">{{ stats.warning }}</span>
                        <span class="badge bg-danger">{{ stats.failure }}</span>
                      </td>
                    </tr>
                  {% endfor %}
                {% endfor %}
              </tbody>
            </table>
          </div>
        </div>
      {% endfor %}
    {% else %}
      <div class="alert alert-info" role="alert">
        <h4 class="alert-heading">No Reports Found</h4>
        Reports should be saved to <code>{{ settings.REPORTS_ROOT }}</code>.
        <hr/>
        <small>This path can be changed by setting <code>REPORTS_ROOT</code> in NetBox's configuration.</small>
      </div>
    {% endif %}
  </div>
{% endblock content-wrapper %}
