{% load helpers %}
<div class="termination{% if obj == termination %} active{% endif %}">
    <strong>{{ termination|linkify }}</strong><br />
    {{ termination|meta:"verbose_name"|bettertitle }}
    {% if termination.type %}
        <small class="text-muted">{{ termination.get_type_display }}</small>
    {% endif %}
</div>
