{% extends 'generic/object.html' %}
{% load helpers %}
{% load plugins %}

{% block breadcrumbs %}
  {{ block.super }}
  <li class="breadcrumb-item">
    <a href="{% url 'dcim:device_devicebays' pk=object.device.pk %}">{{ object.device }}</a>
  </li>
{% endblock %}

{% block content %}
    <div class="row">
        <div class="col col-md-6">
            <div class="card">
                <h5 class="card-header">
                    Device Bay
                </h5>
                <div class="card-body">
                    <table class="table table-hover attr-table">
                        <tr>
                            <th scope="row">Device</th>
                            <td>{{ object.device|linkify }}</td>
                        </tr>
                        <tr>
                            <th scope="row">Name</th>
                            <td>{{ object.name }}</td>
                        </tr>
                        <tr>
                            <th scope="row">Label</th>
                            <td>{{ object.label|placeholder }}</td>
                        </tr>
                        <tr>
                            <th scope="row">Description</th>
                            <td>{{ object.description|placeholder }}</td>
                        </tr>
                    </table>
                </div>
            </div>
        {% include 'inc/panels/custom_fields.html' %}
        {% include 'inc/panels/tags.html' %}
        {% plugin_left_page object %}
        </div>
        <div class="col col-md-6">
            <div class="card">
                <h5 class="card-header">
                    Installed Device
                </h5>
                <div class="card-body">
                {% if object.installed_device %}
                    {% with device=object.installed_device %}
                        <table class="table table-hover attr-table">
                            <tr>
                                <th scope="row">Device</th>
                                <td>{{ device|linkify }}</td>
                            </tr>
                            <tr>
                                <th scope="row">Device Type</th>
                                <td>{{ device.device_type }}</td>
                            </tr>
                        </table>
                    {% endwith %}
                {% else %}
                    <div class="text-muted">
                        None
                    </div>
                {% endif %}
                </div>
            </div>
            {% plugin_right_page object %}
        </div>
    </div>
    <div class="row">
        <div class="col col-md-12">
            {% plugin_full_width_page object %}
        </div>
    </div>
{% endblock %}
