{% extends 'generic/object_edit.html' %}
{% load static %}
{% load form_helpers %}

{% block title %}Bulk Add IP Addresses{% endblock %}

{% block tabs %}
  {% include 'ipam/inc/ipaddress_edit_header.html' with active_tab='bulk_add' %}
{% endblock %}

{% block form %}
    <div class="field-group my-5">
        <div class="row mb-2">
          <h5 class="offset-sm-3">IP Addresses</h5>
        </div>
        {% render_field form.pattern %}
        {% render_field model_form.status %}
        {% render_field model_form.role %}
        {% render_field model_form.vrf %}
        {% render_field model_form.description %}
        {% render_field model_form.tags %}
    </div>

    <div class="field-group my-5">
        <div class="row mb-2">
          <h5 class="offset-sm-3">Tenancy</h5>
        </div>
        {% render_field model_form.tenant_group %}
        {% render_field model_form.tenant %}
    </div>
    {% if model_form.custom_fields %}
        <div class="field-group my-5">
            <div class="row mb-2">
              <h5 class="offset-sm-3">Custom Fields</h5>
            </div>
            {% render_custom_fields model_form %}
        </div>
    {% endif %}
{% endblock %}
