{% extends 'generic/object.html' %}
{% load helpers %}
{% load plugins %}
{% load render_table from django_tables2 %}

{% block breadcrumbs %}
  {{ block.super }}
  {% for contactgroup in object.get_ancestors %}
    <li class="breadcrumb-item"><a href="{% url 'tenancy:contactgroup_list' %}?parent_id={{ contactgroup.pk }}">{{ contactgroup }}</a></li>
  {% endfor %}
{% endblock %}

{% block content %}
  <div class="row mb-3">
    <div class="col col-md-6">
      <div class="card">
        <h5 class="card-header">
          Contact Group
        </h5>
        <div class="card-body">
          <table class="table table-hover attr-table">
            <tr>
              <th scope="row">Name</th>
              <td>{{ object.name }}</td>
            </tr>
            <tr>
              <th scope="row">Description</th>
              <td>{{ object.description|placeholder }}</td>
            </tr>
            <tr>
              <th scope="row">Parent</th>
              <td>{{ object.parent|linkify|placeholder }}</td>
            </tr>
            <tr>
              <th scope="row">Contacts</th>
              <td>
                <a href="{% url 'tenancy:contact_list' %}?group_id={{ object.pk }}">{{ contacts_table.rows|length }}</a>
              </td>
            </tr>
          </table>
        </div>
      </div>
      {% include 'inc/panels/tags.html' %}
      {% plugin_left_page object %}
    </div>
    <div class="col col-md-6">
      {% include 'inc/panels/custom_fields.html' %}
      <div class="card">
        <h5 class="card-header">
          Child Groups
        </h5>
        <div class="card-body table-responsive">
          {% render_table child_groups_table 'inc/table.html' %}
        </div>
        {% if perms.tenancy.add_contactgroup %}
          <div class="card-footer text-end noprint">
            <a href="{% url 'tenancy:contactgroup_add' %}?parent={{ object.pk }}" class="btn btn-sm btn-primary">
              <span class="mdi mdi-plus-thick" aria-hidden="true"></span> Add Contact Group
            </a>
          </div>
        {% endif %}
      </div>
      {% plugin_right_page object %}
    </div>
  </div>
  <div class="row mb-3">
    <div class="col col-md-12">
      <div class="card">
        <h5 class="card-header">Contacts</h5>
        <div class="card-body table-responsive">
          {% render_table contacts_table 'inc/table.html' %}
          {% include 'inc/paginator.html' with paginator=contacts_table.paginator page=contacts_table.page %}
        </div>
      </div>
      {% plugin_full_width_page object %}
    </div>
  </div>
{% endblock %}
