{% extends 'generic/object.html' %}
{% load helpers %}
{% load plugins %}
{% load render_table from django_tables2 %}

{% block breadcrumbs %}
  {{ block.super }}
  {% if object.manufacturer %}
    <li class="breadcrumb-item"><a href="{% url 'dcim:platform_list' %}?manufacturer_id={{ object.manufacturer.pk }}">{{ object.manufacturer }}</a></li>
  {% endif %}
{% endblock %}

{% block extra_controls %}
  {% if perms.dcim.add_device %}
    <a href="{% url 'dcim:device_add' %}?device_role={{ object.pk }}" class="btn btn-sm btn-primary">
      <span class="mdi mdi-plus-thick" aria-hidden="true"></span> Add Device
    </a>
  {% endif %}
{% endblock extra_controls %}

{% block content %}
<div class="row mb-3">
	<div class="col col-md-6">
    <div class="card">
      <h5 class="card-header">
        Platform
      </h5>
      <div class="card-body">
        <table class="table table-hover attr-table">
          <tr>
            <th scope="row">Name</th>
            <td>{{ object.name }}</td>
          </tr>
          <tr>
            <th scope="row">Description</th>
            <td>{{ object.description|placeholder }}</td>
          </tr>
          <tr>
            <th scope="row">Manufacturer</th>
            <td>{{ object.manufacturer|linkify|placeholder }}</td>
          </tr>
          <tr>
            <th scope="row">NAPALM Driver</th>
            <td>{{ object.napalm_driver|placeholder }}</td>
          </tr>
          <tr>
            <th scope="row">Devices</th>
            <td>
              <a href="{% url 'dcim:device_list' %}?platform_id={{ object.pk }}">{{ devices_table.rows|length }}</a>
            </td>
          </tr>
          <tr>
            <th scope="row">Virtual Machines</th>
            <td>
              <a href="{% url 'virtualization:virtualmachine_list' %}?platform_id={{ object.pk }}">{{ virtualmachine_count }}</a>
            </td>
          </tr>
        </table>
      </div>
    </div>
    {% include 'inc/panels/tags.html' %}
    {% plugin_left_page object %}
	</div>
	<div class="col col-md-6">
    <div class="card">
      <h5 class="card-header">
        NAPALM Arguments
      </h5>
      <div class="card-body">
        <pre>{{ object.napalm_args|json }}</pre>
      </div>
    </div>
    {% include 'inc/panels/custom_fields.html' %}
    {% plugin_right_page object %}
  </div>
</div>
<div class="row mb-3">
	<div class="col col-md-12">
    <div class="card">
      <h5 class="card-header">Devices</h5>
      <div class="card-body table-responsive">
        {% render_table devices_table 'inc/table.html' %}
        {% include 'inc/paginator.html' with paginator=devices_table.paginator page=devices_table.page %}
      </div>
    </div>
    {% plugin_full_width_page object %}
  </div>
</div>
{% endblock %}
