{% if url %}
  <button type="submit" name="_delete" formaction="{{ url }}" class="btn btn-danger btn-sm">
    <i class="mdi mdi-trash-can-outline" aria-hidden="true"></i> Delete Selected
  </button>
{% endif %}
