{% extends 'dcim/device/base.html' %}
{% load render_table from django_tables2 %}
{% load buttons %}
{% load static %}
{% load helpers %}
{% load plugins %}

{% block content %}
    <div class="row">
        <div class="col col-md-6">
            <div class="card">
                <h5 class="card-header">
                    Device
                </h5>
                <div class="card-body">
                    <table class="table table-hover attr-table">
                        <tr>
                            <th scope="row">Region</th>
                            <td>
                                {% if object.site.region %}
                                    {% for region in object.site.region.get_ancestors %}
                                        {{ region|linkify }} /
                                    {% endfor %}
                                    {{ object.site.region|linkify }}
                                {% else %}
                                    <span class="text-muted">None</span>
                                {% endif %}
                            </td>
                        </tr>
                        <tr>
                            <th scope="row">Site</th>
                            <td>{{ object.site|linkify }}</td>
                        </tr>
                        <tr>
                            <th scope="row">Location</th>
                            <td>
                            {% if object.location %}
                                {% for location in object.location.get_ancestors %}
                                    {{ location|linkify }} /
                                {% endfor %}
                                {{ object.location|linkify }}
                            {% else %}
                                <span class="text-muted">None</span>
                            {% endif %}
                            </td>
                        </tr>
                        <tr>
                            <th scope="row">Rack</th>
                            <td>
                                {% if object.rack %}
                                    <a href="{% url 'dcim:rack' pk=object.rack.pk %}">{{ object.rack }}</a>
                                {% else %}
                                    <span class="text-muted">None</span>
                                {% endif %}
                            </td>
                        </tr>
                        <tr>
                            <th scope="row">Position</th>
                            <td>
                                {% if object.parent_bay %}
                                    {% with object.parent_bay.device as parent %}
                                        {{ parent|linkify }} / {{ object.parent_bay }}
                                        {% if parent.position %}
                                            (U{{ parent.position }} / {{ parent.get_face_display }})
                                        {% endif %}
                                    {% endwith %}
                                {% elif object.rack and object.position %}
                                    <span>U{{ object.position }} / {{ object.get_face_display }}</span>
                                {% elif object.rack and object.device_type.u_height %}
                                    <span class="badge bg-warning">Not racked</span>
                                {% else %}
                                    <span class="text-muted">&mdash;</span>
                                {% endif %}
                            </td>
                        </tr>
                        <tr>
                            <th scope="row">Tenant</th>
                            <td>
                                {% if object.tenant.group %}
                                    {{ object.tenant.group|linkify }} /
                                {% endif %}
                                {{ object.tenant|linkify|placeholder }}
                            </td>
                        </tr>
                        <tr>
                            <th scope="row">Device Type</th>
                            <td>
                                {{ object.device_type|linkify }} ({{ object.device_type.u_height }}U)
                            </td>
                        </tr>
                        <tr>
                            <td>Airflow</td>
                            <td>
                                {{ object.get_airflow_display|placeholder }}
                            </td>
                        </tr>
                        <tr>
                            <th scope="row">Serial Number</th>
                            <td class="font-monospace">{{ object.serial|placeholder }}</td>
                        </tr>
                        <tr>
                            <th scope="row">Asset Tag</th>
                            <td class="font-monospace">{{ object.asset_tag|placeholder }}</td>
                        </tr>
                    </table>
                </div>
            </div>
            {% if vc_members %}
                <div class="card">
                    <h5 class="card-header">
                        Virtual Chassis
                    </h5>
                    <div class="card-body">
                        <table class="table table-hover attr-table">
                            <tr>
                                <th>Device</th>
                                <th>Position</th>
                                <th>Master</th>
                                <th>Priority</th>
                            </tr>
                            {% for vc_member in vc_members %}
                                <tr{% if vc_member == object %} class="info"{% endif %}>
                                    <td>
                                        {{ vc_member|linkify }}
                                    </td>
                                    <td>
                                      {% badge vc_member.vc_position show_empty=True %}
                                    </td>
                                    <td>
                                      {% if object.virtual_chassis.master == vc_member %}<i class="mdi mdi-check-bold"></i>{% endif %}
                                    </td>
                                    <td>
                                      {{ vc_member.vc_priority|default:"" }}
                                    </td>
                                </tr>
                            {% endfor %}
                        </table>
                    </div>
                    <div class="card-footer text-end noprint">
                        <a href="{{ object.virtual_chassis.get_absolute_url }}" class="btn btn-primary btn-sm">
                            <span class="mdi mdi-arrow-right-bold" aria-hidden="true"></span> View Virtual Chassis
                        </a>
                    </div>
                </div>
            {% endif %}
            {% include 'inc/panels/custom_fields.html' %}
            {% include 'inc/panels/tags.html' %}
            {% include 'inc/panels/comments.html' %}
            {% plugin_left_page object %}
        </div>
        <div class="col col-md-6">
            <div class="card">
                <h5 class="card-header">
                    Management
                </h5>
                <div class="card-body">
                    <table class="table table-hover attr-table">
                        <tr>
                            <th scope="row">Status</th>
                            <td>{% badge object.get_status_display bg_color=object.get_status_color %}</td>
                        </tr>
                        <tr>
                            <th scope="row">Role</th>
                            <td>
                                <a href="{% url 'dcim:device_list' %}?role={{ object.device_role.slug }}">{{ object.device_role }}</a>
                            </td>
                        </tr>
                        <tr>
                            <th scope="row">Platform</th>
                            <td>{{ object.platform|linkify|placeholder }}</td>
                        </tr>
                        <tr>
                            <th scope="row">Primary IPv4</th>
                            <td>
                              {% if object.primary_ip4 %}
                                <a href="{% url 'ipam:ipaddress' pk=object.primary_ip4.pk %}">{{ object.primary_ip4.address.ip }}</a>
                                {% if object.primary_ip4.nat_inside %}
                                  (NAT for {{ object.primary_ip4.nat_inside.address.ip|linkify }})
                                {% elif object.primary_ip4.nat_outside %}
                                  (NAT: {{ object.primary_ip4.nat_outside.address.ip|linkify }})
                                {% endif %}
                              {% else %}
                                <span class="text-muted">&mdash;</span>
                              {% endif %}
                            </td>
                        </tr>
                        <tr>
                            <th scope="row">Primary IPv6</th>
                            <td>
                              {% if object.primary_ip6 %}
                                <a href="{% url 'ipam:ipaddress' pk=object.primary_ip6.pk %}">{{ object.primary_ip6.address.ip }}</a>
                                {% if object.primary_ip6.nat_inside %}
                                  (NAT for {{ object.primary_ip6.nat_inside.address.ip|linkify }})
                                {% elif object.primary_ip6.nat_outside %}
                                  (NAT: {{ object.primary_ip6.nat_outside.address.ip|linkify }})
                                {% endif %}
                              {% else %}
                                <span class="text-muted">&mdash;</span>
                              {% endif %}
                            </td>
                        </tr>
                        {% if object.cluster %}
                            <tr>
                                <th>Cluster</th>
                                <td>
                                    {% if object.cluster.group %}
                                        {{ object.cluster.group|linkify }} /
                                    {% endif %}
                                    {{ object.cluster|linkify }}
                                </td>
                            </tr>
                        {% endif %}
                    </table>
                </div>
            </div>
            {% if object.powerports.exists and object.poweroutlets.exists %}
                <div class="card">
                    <h5 class="card-header">
                        Power Utilization
                    </h5>
                    <div class="card-body">
                        <table class="table table-hover">
                            <tr>
                                <th>Input</th>
                                <th>Outlets</th>
                                <th>Allocated</th>
                                <th>Available</th>
                                <th>Utilization</th>
                            </tr>
                            {% for powerport in object.powerports.all %}
                                {% with utilization=powerport.get_power_draw powerfeed=powerport.connected_endpoint %}
                                    <tr>
                                        <td>{{ powerport }}</td>
                                        <td>{{ utilization.outlet_count }}</td>
                                        <td>{{ utilization.allocated }}VA</td>
                                        {% if powerfeed.available_power %}
                                            <td>{{ powerfeed.available_power }}VA</td>
                                            <td>{% utilization_graph utilization.allocated|percentage:powerfeed.available_power %}</td>
                                        {% else %}
                                            <td class="text-muted">&mdash;</td>
                                            <td class="text-muted">&mdash;</td>
                                        {% endif %}
                                    </tr>
                                    {% for leg in utilization.legs %}
                                        <tr>
                                            <td style="padding-left: 20px">Leg {{ leg.name }}</td>
                                            <td>{{ leg.outlet_count }}</td>
                                            <td>{{ leg.allocated }}</td>
                                            <td>{{ powerfeed.available_power|divide:3 }}VA</td>
                                            {% with phase_available=powerfeed.available_power|divide:3 %}
                                                <td>{% utilization_graph leg.allocated|percentage:phase_available %}</td>
                                            {% endwith %}
                                        </tr>
                                    {% endfor %}
                                {% endwith %}
                            {% endfor %}
                        </table>
                    </div>
                </div>
            {% endif %}
            <div class="card">
                <h5 class="card-header">
                    Services
                </h5>
                <div class="card-body">
                {% if services %}
                    <table class="table table-hover">
                        {% for service in services %}
                            {% include 'ipam/inc/service.html' %}
                        {% endfor %}
                    </table>
                {% else %}
                    <div class="text-muted">
                        None
                    </div>
                {% endif %}
                </div>
                {% if perms.ipam.add_service %}
                <div class="card-footer text-end noprint">
                    <a href="{% url 'ipam:service_add' %}?device={{ object.pk }}" class="btn btn-sm btn-primary">
                        <span class="mdi mdi-plus-thick" aria-hidden="true"></span> Assign Service
                    </a>
                </div>
                {% endif %}
            </div>
            {% include 'inc/panels/contacts.html' %}
            {% include 'inc/panels/image_attachments.html' %}
            {% plugin_right_page object %}
        </div>
    </div>
    <div class="row">
        <div class="col col-md-12">
            {% plugin_full_width_page object %}
        </div>
    </div>
{% endblock %}
