{% extends 'generic/object.html' %}
{% load helpers %}
{% load plugins %}

{% block content %}
  <div class="row">
    <div class="col col-md-6">
      <div class="card">
        <h5 class="card-header">Interface A</h5>
        <div class="card-body">
          {% include 'wireless/inc/wirelesslink_interface.html' with interface=object.interface_a %}
        </div>
      </div>
      <div class="card">
        <h5 class="card-header">Link Properties</h5>
        <div class="card-body">
          <table class="table table-hover attr-table">
            <tr>
              <th scope="row">Status</th>
              <td>{% badge object.get_status_display bg_color=object.get_status_color %}</td>
            </tr>
            <tr>
              <th scope="row">SSID</th>
              <td>{{ object.ssid|placeholder }}</td>
            </tr>
            <tr>
              <th scope="row">Description</th>
              <td>{{ object.description|placeholder }}</td>
            </tr>
          </table>
        </div>
      </div>
      {% include 'inc/panels/tags.html' %}
      {% plugin_left_page object %}
    </div>
    <div class="col col-md-6">
      <div class="card">
        <h5 class="card-header">Interface B</h5>
        <div class="card-body">
          {% include 'wireless/inc/wirelesslink_interface.html' with interface=object.interface_b %}
        </div>
      </div>
      {% include 'wireless/inc/authentication_attrs.html' %}
      {% include 'inc/panels/custom_fields.html' %}
      {% plugin_right_page object %}
    </div>
  </div>
  <div class="row">
    <div class="col col-md-12">
      {% plugin_full_width_page object %}
    </div>
  </div>
{% endblock %}
