{% extends 'ipam/iprange/base.html' %}
{% load helpers %}
{% load plugins %}

{% block content %}
<div class="row">
    <div class="col col-md-6">
        <div class="card">
            <h5 class="card-header">
              IP Range
            </h5>
            <div class="card-body">
              <table class="table table-hover attr-table">
                <tr>
                    <th scope="row">Family</th>
                    <td>IPv{{ object.family }}</td>
                </tr>
                <tr>
                    <th scope="row">Starting Address</th>
                    <td>{{ object.start_address }}</td>
                </tr>
                <tr>
                    <th scope="row">Ending Address</th>
                    <td>{{ object.end_address }}</td>
                </tr>
                <tr>
                    <th scope="row">Size</th>
                    <td>{{ object.size }}</td>
                </tr>
                <tr>
                    <th scope="row">Utilization</th>
                    <td>
                      {% utilization_graph object.utilization %}
                    </td>
                </tr>
                <tr>
                    <th scope="row">VRF</th>
                    <td>
                        {% if object.vrf %}
                            {{ object.vrf|linkify }} ({{ object.vrf.rd }})
                        {% else %}
                            <span>Global</span>
                        {% endif %}
                    </td>
                </tr>
                <tr>
                    <th scope="row">Role</th>
                    <td>{{ object.role|linkify|placeholder }}</td>
                </tr>
                <tr>
                    <th scope="row">Status</th>
                    <td>{% badge object.get_status_display bg_color=object.get_status_color %}</td>
                </tr>
                <tr>
                    <th scope="row">Tenant</th>
                    <td>
                        {% if object.tenant.group %}
                            {{ object.tenant.group|linkify }} /
                        {% endif %}
                        {{ object.tenant|linkify|placeholder }}
                    </td>
                </tr>
                <tr>
                    <th scope="row">Description</th>
                    <td>{{ object.description|placeholder }}</td>
                </tr>
              </table>
            </div>
        </div>
        {% plugin_left_page object %}
    </div>
    <div class="col col-md-6">
        {% include 'inc/panels/tags.html' %}
        {% include 'inc/panels/custom_fields.html' %}
        {% plugin_right_page object %}
    </div>
</div>
<div class="row">
    <div class="col col-md-12">
        {% plugin_full_width_page object %}
    </div>
</div>
{% endblock %}
