{% extends 'dcim/component_template_create.html' %}
{% load form_helpers %}

{% block replication_fields %}
  {{ block.super }}
  {% render_field replication_form.rear_port_set %}
{% endblock replication_fields %}
