{% extends "admin/base_site.html" %}

{% block title %}Installed Plugins {{ block.super }}{% endblock %}

{% block breadcrumbs %}
    <div class="breadcrumbs">
        <a href="{% url 'admin:index' %}">Home</a> &rsaquo;
        <a href="{% url 'plugins_list' %}">Installed Plugins</a>
    </div>
{% endblock %}

{% block content_title %}<h1>Installed Plugins{{ queue.name }}</h1>{% endblock %}

{% block content %}
<div id="content-main">
    <div class="module" id="changelist">
        <div class="results">
            <table id="result_list">
                <thead>
                    <tr>
                        <th><div class="text"><span>Name</span></div></th>
                        <th><div class="text"><span>Package Name</span></div></th>
                        <th><div class="text"><span>Author</span></div></th>
                        <th><div class="text"><span>Author Email</span></div></th>
                        <th><div class="text"><span>Description</span></div></th>
                        <th><div class="text"><span>Version</span></div></th>
                    </tr>
                </thead>
                <tbody>
                    {% for plugin in plugins %}
                        <tr class="{% cycle 'row1' 'row2' %}">
                            <td>
                                {{ plugin.verbose_name }}
                            </td>
                            <td>
                                {{ plugin.name }}
                            </td>
                            <td>
                                {{ plugin.author }}
                            </td>
                            <td>
                                {{ plugin.author_email }}
                            </td>
                            <td>
                                {{ plugin.description }}
                            </td>
                            <td>
                                {{ plugin.version }}
                            </td>
                        </tr>
                    {% endfor %}
                </tbody>
            </table>
        </div>
    </div>
</div>
{% endblock %}
