{% extends 'base/layout.html' %}
{% load render_table from django_tables2 %}

{% block title %}Import Completed{% endblock %}

{% block content %}
    <div class="table-responsive">
        {% render_table table 'inc/table.html' %}
    </div>
    {% if return_url %}
        <a href="{{ return_url }}" class="btn btn-outline-dark">View All</a>
    {% endif %}
    <a href="{{ request.path }}" class="btn btn-primary">
        <span class="mdi mdi-database-import-outline" aria-hidden="true"></span>
        Import More
    </a>
{% endblock content %}
