{% extends 'generic/object.html' %}
{% load helpers %}
{% load plugins %}
{% load tz %}

{% block breadcrumbs %}
  {{ block.super }}
  {% if object.region %}
    {% for region in object.region.get_ancestors %}
      <li class="breadcrumb-item"><a href="{% url 'dcim:site_list' %}?region_id={{ region.pk }}">{{ region }}</a></li>
    {% endfor %}
    <li class="breadcrumb-item"><a href="{% url 'dcim:site_list' %}?region_id={{ object.region.pk }}">{{ object.region }}</a></li>
  {% elif object.group %}
    {% for group in object.group.get_ancestors %}
      <li class="breadcrumb-item"><a href="{% url 'dcim:site_list' %}?group_id={{ group.pk }}">{{ group }}</a></li>
    {% endfor %}
    <li class="breadcrumb-item"><a href="{% url 'dcim:site_list' %}?group_id={{ object.group.pk }}">{{ object.group }}</a></li>
  {% endif %}
{% endblock %}

{% block content %}
<div class="row">
	<div class="col col-md-6">
    <div class="card">
      <h5 class="card-header">Site</h5>
      <div class="card-body">
        <table class="table table-hover attr-table">
          <tr>
            <th scope="row">Region</th>
            <td>
              {% if object.region %}
                {% for region in object.region.get_ancestors %}
                  {{ region|linkify }} /
                {% endfor %}
                {{ object.region|linkify }}
              {% else %}
                <span class="text-muted">None</span>
              {% endif %}
            </td>
          </tr>
          <tr>
            <th scope="row">Group</th>
            <td>
              {% if object.group %}
                {% for group in object.group.get_ancestors %}
                  {{ group|linkify }} /
                {% endfor %}
                {{ object.group|linkify }}
              {% else %}
                <span class="text-muted">None</span>
              {% endif %}
            </td>
          </tr>
          <tr>
            <th scope="row">Status</th>
            <td>{% badge object.get_status_display bg_color=object.get_status_color %}</td>
          </tr>
          <tr>
            <th scope="row">Tenant</th>
            <td>
              {% if object.tenant.group %}
                {{ object.tenant.group|linkify }} /
              {% endif %}
              {{ object.tenant|linkify|placeholder }}
            </td>
          </tr>
          <tr>
            <th scope="row">Facility</th>
            <td>{{ object.facility|placeholder }}</td>
          </tr>
          <tr>
            <th scope="row">Description</th>
            <td>{{ object.description|placeholder }}</td>
          </tr>
          <tr>
            <th scope="row">Time Zone</th>
            <td>
              {% if object.time_zone %}
                {{ object.time_zone }} (UTC {{ object.time_zone|tzoffset }})<br />
                <small class="text-muted">Site time: {% timezone object.time_zone %}{% annotated_now %}{% endtimezone %}</small>
              {% else %}
                <span class="text-muted">&mdash;</span>
              {% endif %}
            </td>
          </tr>
          <tr>
            <th scope="row">Physical Address</th>
            <td>
              {% if object.physical_address %}
                <div class="float-end noprint">
                  <a href="{{ config.MAPS_URL }}{{ object.physical_address|urlencode }}" target="_blank" class="btn btn-primary btn-sm">
                    <i class="mdi mdi-map-marker"></i> Map It
                  </a>
                </div>
                <span>{{ object.physical_address|linebreaksbr }}</span>
              {% else %}
                <span class="text-muted">&mdash;</span>
              {% endif %}
            </td>
          </tr>
          <tr>
            <th scope="row">Shipping Address</th>
            <td>{{ object.shipping_address|linebreaksbr|placeholder }}</td>
          </tr>
          <tr>
            <th scope="row">GPS Coordinates</th>
            <td>
              {% if object.latitude and object.longitude %}
                <div class="float-end noprint">
                  <a href="{{ config.MAPS_URL }}{{ object.latitude }},{{ object.longitude }}" target="_blank" class="btn btn-primary btn-sm">
                    <i class="mdi mdi-map-marker"></i> Map It
                  </a>
                </div>
                <span>{{ object.latitude }}, {{ object.longitude }}</span>
              {% else %}
                <span class="text-muted">&mdash;</span>
              {% endif %}
            </td>
          </tr>
        </table>
      </div>
    </div>
    {% include 'inc/panels/custom_fields.html' %}
    {% include 'inc/panels/tags.html' %}
    {% include 'inc/panels/comments.html' %}
    {% plugin_left_page object %}
    </div>
    <div class="col col-md-6">
      <div class="card">
        <h5 class="card-header">Related Objects</h5>
        <div class="card-body">
          <table class="table table-hover attr-table">
            <tr>
              <th scope="row">Locations</th>
              <td class="text-end">
                {% if stats.location_count %}
                  <a href="{% url 'dcim:location_list' %}?site_id={{ object.pk }}">{{ stats.location_count }}</a>
                {% else %}
                  {{ ''|placeholder }}
                {% endif %}
              </td>
            </tr>
            <tr>
              <th scope="row">Racks</th>
              <td class="text-end">
                {% if stats.rack_count %}
                  <div class="dropdown">
                    <button class="btn btn-sm btn-light dropdown-toggle" type="button" data-bs-toggle="dropdown" aria-expanded="false">
                      {{ stats.rack_count }}
                    </button>
                    <ul class="dropdown-menu">
                      <li><a class="dropdown-item" href="{% url 'dcim:rack_list' %}?site_id={{ object.pk }}">View Racks</a></li>
                      <li><a class="dropdown-item" href="{% url 'dcim:rack_elevation_list' %}?site_id={{ object.pk }}">View Elevations</a></li>
                    </ul>
                  </div>
                {% else %}
                  {{ ''|placeholder }}
                {% endif %}
              </td>
            </tr>
            <tr>
              <th scope="row">Devices</th>
              <td class="text-end">
                {% if stats.device_count %}
                  <a href="{% url 'dcim:device_list' %}?site_id={{ object.pk }}">{{ stats.device_count }}</a>
                {% else %}
                  {{ ''|placeholder }}
                {% endif %}
              </td>
            </tr>
            <tr>
              <th scope="row">Virtual Machines</th>
              <td class="text-end">
                {% if stats.vm_count %}
                  <a href="{% url 'virtualization:virtualmachine_list' %}?site_id={{ object.pk }}">{{ stats.vm_count }}</a>
                {% else %}
                  {{ ''|placeholder }}
                {% endif %}
              </td>
            </tr>
            <tr>
              <th scope="row">Prefixes</th>
              <td class="text-end">
                {% if stats.prefix_count %}
                  <a href="{% url 'ipam:prefix_list' %}?site_id={{ object.pk }}">{{ stats.prefix_count }}</a>
                {% else %}
                  {{ ''|placeholder }}
                {% endif %}
              </td>
            </tr>
            <tr>
              <th scope="row">VLAN Groups</th>
              <td class="text-end">
                {% if stats.vlangroup_count %}
                  <a href="{% url 'ipam:vlangroup_list' %}?site={{ object.pk }}">{{ stats.vlangroup_count }}</a>
                {% else %}
                  {{ ''|placeholder }}
                {% endif %}
              </td>
            </tr>
            <tr>
              <th scope="row">VLANs</th>
              <td class="text-end">
                {% if stats.vlan_count %}
                  <a href="{% url 'ipam:vlan_list' %}?site_id={{ object.pk }}">{{ stats.vlan_count }}</a>
                {% else %}
                  {{ ''|placeholder }}
                {% endif %}
              </td>
            </tr>
            <tr>
              <th scope="row">ASNs</th>
              <td class="text-end">
                {% if stats.asn_count %}
                  <a href="{% url 'ipam:asn_list' %}?site_id={{ object.pk }}">{{ stats.asn_count }}</a>
                {% else %}
                  {{ ''|placeholder }}
                {% endif %}
              </td>
            </tr>
            <tr>
              <th scope="row">Circuits</th>
              <td class="text-end">
                {% if stats.circuit_count %}
                <a href="{% url 'circuits:circuit_list' %}?site_id={{ object.pk }}">{{ stats.circuit_count }}</a>
                {% else %}
                  {{ ''|placeholder }}
                {% endif %}
              </td>
            </tr>
          </table>
        </div>
      </div>
      {% include 'dcim/inc/nonracked_devices.html' %}
      {% include 'inc/panels/contacts.html' %}
      <div class="card">
        <h5 class="card-header">Locations</h5>
        <div class='card-body'>
          {% if locations %}
            <table class="table table-hover">
              <tr>
                <th>Location</th>
                <th>Racks</th>
                <th>Devices</th>
                <th></th>
              </tr>
              {% for location in locations %}
                <tr>
                  <td>
                    {% for i in location.level|as_range %}<i class="mdi mdi-circle-small"></i>{% endfor %}
                    {{ location|linkify }}
                  </td>
                  <td>
                    <a href="{% url 'dcim:rack_list' %}?location_id={{ location.pk }}">{{ location.rack_count }}</a>
                  </td>
                  <td>
                    <a href="{% url 'dcim:device_list' %}?location_id={{ location.pk }}">{{ location.device_count }}</a>
                  </td>
                  <td class="text-end noprint">
                    <a href="{% url 'dcim:rack_elevation_list' %}?location_id={{ location.pk }}" class="btn btn-sm btn-primary" title="View Elevations">
                      <i class="mdi mdi-server"></i>
                    </a>
                  </td>
                </tr>
              {% endfor %}
            </table>
          {% else %}
            <span class="text-muted">None</span>
          {% endif %}
        </div>
        {% if perms.dcim.add_location %}
          <div class="card-footer text-end noprint">
            <a href="{% url 'dcim:location_add' %}?site={{ object.pk }}&return_url={{ object.get_absolute_url }}" class="btn btn-primary btn-sm">
              <i class="mdi mdi-plus-thick" aria-hidden="true"></i> Add a location
            </a>
          </div>
        {% endif %}
      </div>
      <div class="card">
        <h5 class="card-header">ASNs</h5>
        <div class='card-body'>
          {% if asns %}
            <table class="table table-hover">
              <tr>
                <th>ASN</th>
                <th>Description</th>
              </tr>
            {% for asn in asns %}
              <tr>
                <td>{{ asn|linkify }}</td>
                <td>{{ asn.description|placeholder }}</td>
              </tr>
            {% endfor %}
            </table>
          {% else %}
            <span class="text-muted">None</span>
          {% endif %}
        </div>
        {% if perms.ipam.add_asn %}
          <div class="card-footer text-end noprint">
            <a href="{% url 'ipam:asn_add' %}?sites={{ object.pk }}&return_url={{ object.get_absolute_url }}" class="btn btn-primary btn-sm">
              <i class="mdi mdi-plus-thick" aria-hidden="true"></i> Add an ASN
            </a>
          </div>
        {% endif %}
      </div>
      {% include 'inc/panels/image_attachments.html' %}
      {% plugin_right_page object %}
	</div>
</div>
<div class="row">
  <div class="col col-md-12">
    {% plugin_full_width_page object %}
  </div>
</div>
{% endblock %}
