{% extends 'generic/object.html' %}
{% load helpers %}
{% load plugins %}

{% block content %}
  <div class="row">
    <div class="col col-md-6">
      <div class="card">
        <h5 class="card-header">Route Target</h5>
        <div class="card-body">
          <table class="table table-hover attr-table">
            <tr>
              <th scope="row">Name</th>
              <td class="font-monospace">{{ object.name }}</td>
            </tr>
            <tr>
              <th  scope="row">Tenant</th>
              <td>{{ object.tenant|linkify|placeholder }}</td>
            </tr>
            <tr>
              <th scope="row">Description</th>
              <td>{{ object.description|placeholder }}</td>
            </tr>
          </table>
        </div>
      </div>
      {% include 'inc/panels/tags.html' %}
      {% include 'inc/panels/custom_fields.html' %}
      {% plugin_left_page object %}
    </div>
    <div class="col col-md-6">
      <div class="mb-4">
        {% include 'inc/panel_table.html' with table=importing_vrfs_table heading="Importing VRFs" %}
      </div>
      {% include 'inc/panel_table.html' with table=exporting_vrfs_table heading="Exporting VRFs" %}
      {% plugin_right_page object %}
    </div>
  </div>
  <div class="row">
    <div class="col col-md-12">
      {% plugin_full_width_page object %}
    </div>
  </div>
{% endblock %}
