{% extends 'base/layout.html' %}

{% block tabs %}
  <ul class="nav nav-tabs px-3">
    <li role="presentation" class="nav-item">
      <a class="nav-link{% if active_tab == 'profile' %} active{% endif %}" href="{% url 'user:profile' %}">Profile</a>
    </li>
    <li role="presentation" class="nav-item">
      <a class="nav-link{% if active_tab == 'preferences' %} active{% endif %}" href="{% url 'user:preferences' %}">Preferences</a>
    </li>
    {% if not request.user.ldap_username %}
      <li role="presentation" class="nav-item">
        <a class="nav-link{% if active_tab == 'password' %} active{% endif %}" href="{% url 'user:change_password' %}">Password</a>
      </li>
    {% endif %}
    <li role="presentation" class="nav-item">
      <a class="nav-link{% if active_tab == 'api-tokens' %} active{% endif %}" href="{% url 'user:token_list' %}">API Tokens</a>
    </li>
  </ul>
{% endblock %}

{% block content-wrapper %}
  <div class="tab-content">
    {% block content %}{% endblock %}
  </div>
{% endblock %}
