{% extends 'virtualization/cluster/base.html' %}
{% load helpers %}
{% load render_table from django_tables2 %}

{% block content %}
  {% include 'inc/table_controls_htmx.html' with table_modal="DeviceTable_config" %}
  
  <form action="{% url 'virtualization:cluster_remove_devices' pk=object.pk %}" method="post">
    {% csrf_token %}
    <div class="card">
      <div class="card-body" id="object_list">
        {% include 'htmx/table.html' %}
      </div>
    </div>
    <div class="noprint bulk-buttons">
      <div class="bulk-button-group">
        {% if perms.virtualization.change_cluster %}
          <button type="submit" name="_remove" class="btn btn-danger btn-sm">
            <span class="mdi mdi-trash-can-outline" aria-hidden="true"></span> Remove Devices
          </button>
        {% endif %}
      </div>
    </div>
  </form>
{% endblock content %}

{% block modals %}
  {{ block.super }}
  {% table_config_form table %}
{% endblock modals %}
