# Generated by Django 3.2.10 on 2021-12-23 15:24

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dcim', '0145_site_remove_deprecated_fields'),
        ('ipam', '0053_asn_model'),
    ]

    operations = [
        migrations.AddField(
            model_name='vlangroup',
            name='max_vid',
            field=models.PositiveSmallIntegerField(default=4094, validators=[django.core.validators.MinValueValidator(1), django.core.validators.MaxValueValidator(4094)]),
        ),
        migrations.AddField(
            model_name='vlangroup',
            name='min_vid',
            field=models.PositiveSmallIntegerField(default=1, validators=[django.core.validators.MinValueValidator(1), django.core.validators.MaxValueValidator(4094)]),
        ),
    ]
