{% load helpers %}

<div class="row mb-3 justify-content-between">
  <div class="table-controls noprint col col-12 col-md-8 col-lg-4">
    <div class="input-group input-group-sm">
      <input
        type="text"
        name="q"
        class="form-control"
        placeholder="Quick search"
        hx-get="{{ request.full_path }}"
        hx-target="#object_list"
        hx-trigger="keyup changed delay:500ms"
      />
    </div>
  </div>
  <div class="table-controls noprint col col-md-3 mb-0">
    {% if request.user.is_authenticated and table_modal %}
      <div class="table-configure input-group input-group-sm">
        <button
          type="button"
          data-bs-toggle="modal"
          title="Configure Table"
          data-bs-target="#{{ table_modal }}"
          class="btn btn-sm btn-outline-dark"
        >
          <i class="mdi mdi-cog"></i> Configure Table
        </button>
      </div>
    {% endif %}
  </div>
</div>
