{% extends 'generic/object_list.html' %}

{% block title %}Change Log{% endblock %}

{% block content-wrapper %}
  {{ block.super }}
  <div class="text-muted px-3">
    Change log retention: {% if config.CHANGELOG_RETENTION %}{{ config.CHANGELOG_RETENTION }} days{% else %}Indefinite{% endif %}
  </div>
{% endblock %}
