{% extends 'generic/object_list.html' %}
{% load buttons %}
{% load helpers %}
{% load render_table from django_tables2 %}

{% block title %}{{ title }}{% endblock %}

{% block content-wrapper %}
  <div class="tab-content">

    {# Connections list #}
    <div class="tab-pane show active" id="object-list" role="tabpanel" aria-labelledby="object-list-tab">
      {% include 'inc/table_controls_htmx.html' %}
      <div class="card">
        <div class="card-body" id="object_list">
          {% include 'htmx/table.html' %}
        </div>
      </div>
    </div>

    {# Filter form #}
    {% if filter_form %}
      <div class="tab-pane show" id="filters-form" role="tabpanel" aria-labelledby="filters-form-tab">
        {% include 'inc/filter_list.html' %}
      </div>
    {% endif %}

  </div>
{% endblock content-wrapper %}
