# Generated by Django 3.2.8 on 2021-10-22 20:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('extras', '0062_clear_secrets_changelog'),
    ]

    operations = [
        migrations.AddField(
            model_name='webhook',
            name='conditions',
            field=models.JSONField(blank=True, null=True),
        ),
    ]
