{% extends base_template %}
{% load form_helpers %}
{% load render_table from django_tables2 %}

{% render_errors form %}

{% block content %}
  {% if perms.extras.add_journalentry %}
    <form action="{% url 'extras:journalentry_add' %}" method="post" enctype="multipart/form-data">
      <div class="container">
        <div class="field-group">
          <h4>New Journal Entry</h4>
          {% csrf_token %}
          {% render_form form %}
        </div>
        <div class="col col-md-12 text-end my-3">
          <a href="{{ object.get_absolute_url }}" class="btn btn-outline-danger">Cancel</a>
          <button type="submit" class="btn btn-primary">Save</button>
        </div>
      </div>
    </form>
  {% endif %}
  <div class="card">
    <div class="card-body table-responsive">
      {% render_table table 'inc/table.html' %}
      {% include 'inc/paginator.html' with paginator=table.paginator page=table.page %}
    </div>
  </div>
{% endblock %}
