{% extends 'generic/object_edit.html' %}
{% load form_helpers %}

{% block form %}
  <div class="field-group my-5">
    <div class="row mb-2">
      <h5 class="offset-sm-3">FHRP Group Assignment</h5>
    </div>
    <div class="row mb-3">
        <label class="col-sm-3 col-form-label text-lg-end">Interface</label>
        <div class="col">
            <input class="form-control" value="{{ form.instance.interface }}" disabled />
        </div>
    </div>
    {% render_field form.group %}
    {% render_field form.priority %}
  </div>
{% endblock %}
