{% extends 'base/layout.html' %}
{% load helpers %}
{% load render_table from django_tables2 %}

{% comment %}
Blocks:
  message_extra: Supplementary warning message content

Context:
  model: The model class of the objects being deleted
  form: The bulk delete form class
  table: The table class for rendering list of objects being deleted
  return_url: The URL to which the user is redirected after submitting the form
{% endcomment %}

{% block title %}Delete {{ table.rows|length }} {{ model|meta:"verbose_name_plural"|bettertitle }}?{% endblock %}

{% block content %}
  <div class="container-md px-0">
    <div class="alert alert-danger mb-3" role="alert">
      <h4 class="alert-heading">Confirm Bulk Deletion</h4>
      <hr />
      <strong>Warning:</strong> The following operation will delete <strong>{{ table.rows|length }}</strong>
      {{ model|meta:"verbose_name_plural" }}. Please carefully review the objects to be deleted and confirm below.
      {% block message_extra %}{% endblock %}
    </div>
  </div>
  <div class="container-xl px-0">
    <div class="table-responsive">
      {% render_table table 'inc/table.html' %}
    </div>
    <div class="row mt-3">
      <form action="" method="post">
        {% csrf_token %}
        {% for field in form.hidden_fields %}
          {{ field }}
        {% endfor %}
        <div class="text-end">
          <a href="{{ return_url }}" class="btn btn-outline-dark">Cancel</a>
          <button type="submit" name="_confirm" class="btn btn-danger">Delete {{ table.rows|length }} {{ model|meta:"verbose_name_plural" }}</button>
        </div>
      </form>
    </div>
  </div>
{% endblock content %}
