{% extends 'dcim/device/base.html' %}
{% load render_table from django_tables2 %}
{% load helpers %}
{% load static %}

{% block content %}
  {% include 'inc/table_controls_htmx.html' with table_modal="DeviceInventoryItemTable_config" %}
  
  <form method="post">
    {% csrf_token %}

    <div class="card">
      <div class="card-body" id="object_list">
        {% include 'htmx/table.html' %}
      </div>
    </div>

    <div class="noprint bulk-buttons">
        <div class="bulk-button-group">
            {% if perms.dcim.change_inventoryitem %}
                <button type="submit" name="_rename" formaction="{% url 'dcim:inventoryitem_bulk_rename' %}?return_url={% url 'dcim:device_inventory' pk=object.pk %}" class="btn btn-warning btn-sm">
                    <i class="mdi mdi-pencil-outline" aria-hidden="true"></i> Rename
                </button>
                <button type="submit" name="_edit" formaction="{% url 'dcim:inventoryitem_bulk_edit' %}?device={{ object.pk }}&return_url={% url 'dcim:device_inventory' pk=object.pk %}" class="btn btn-warning btn-sm">
                    <i class="mdi mdi-pencil" aria-hidden="true"></i> Edit
                </button>
            {% endif %}
            {% if perms.dcim.delete_inventoryitem %}
                <button type="submit" name="_delete" formaction="{% url 'dcim:inventoryitem_bulk_delete' %}?return_url={% url 'dcim:device_inventory' pk=object.pk %}" class="btn btn-danger btn-sm">
                    <i class="mdi mdi-trash-can-outline" aria-hidden="true"></i> Delete
                </button>
            {% endif %}
        </div>
        {% if perms.dcim.add_inventoryitem %}
            <div class="bulk-button-group">
                <a href="{% url 'dcim:inventoryitem_add' %}?device={{ object.pk }}&return_url={% url 'dcim:device_inventory' pk=object.pk %}" class="btn btn-primary btn-sm">
                    <i class="mdi mdi-plus-thick" aria-hidden="true"></i> Add Inventory Item
                </a>
            </div>
        {% endif %}
    </div>
  </form>
{% endblock %}

{% block modals %}
  {{ block.super }}
  {% table_config_form table %}
{% endblock modals %}
