{% extends 'dcim/component_create.html' %}
{% load helpers %}
{% load form_helpers %}

{% block replication_fields %}
  {{ block.super }}
  {% if object.component %}
    <div class="row mb-3">
        <label class="col-sm-3 col-form-label text-lg-end">
          {{ object.component|meta:"verbose_name"|bettertitle }}
        </label>
        <div class="col">
            <input class="form-control" value="{{ object.component }}" disabled />
        </div>
    </div>
  {% endif %}
{% endblock replication_fields %}
