{% extends 'generic/object.html' %}
{% load static %}
{% load helpers %}
{% load plugins %}
{% load render_table from django_tables2 %}

{% block breadcrumbs %}
  {{ block.super }}
  <li class="breadcrumb-item"><a href="{% url 'circuits:providernetwork_list' %}?provider_id={{ object.provider_id }}">{{ object.provider }}</a></li>
{% endblock %}

{% block content %}
<div class="row mb-3">
	  <div class="col col-md-6">
        <div class="card">
            <h5 class="card-header">
                Provider Network
            </h5>
            <div class="card-body">
                <table class="table table-hover attr-table">
                    <tr>
                        <th scope="row">Provider</th>
                        <td>{{ object.provider|linkify }}</td>
                    </tr>
                    <tr>
                        <th scope="row">Name</th>
                        <td>{{ object.name }}</td>
                    </tr>
                    <tr>
                        <th scope="row">Service ID</th>
                        <td>{{ object.service_id|placeholder }}</td>
                    </tr>
                    <tr>
                        <th scope="row">Description</th>
                        <td>{{ object.description|placeholder }}</td>
                    </tr>
                </table>
            </div>
        </div>
        {% plugin_left_page object %}
    </div>
    <div class="col col-md-6">
        {% include 'inc/panels/custom_fields.html' %}
        {% include 'inc/panels/tags.html' %}
        {% include 'inc/panels/comments.html' %}
        {% plugin_right_page object %}
    </div>
</div>
<div class="row mb-3">
  <div class="col col-md-12">
    <div class="card">
      <h5 class="card-header">Circuits</h5>
      <div class="card-body table-responsive">
        {% render_table circuits_table 'inc/table.html' %}
        {% include 'inc/paginator.html' with paginator=circuits_table.paginator page=circuits_table.page %}
      </div>
    </div>
    {% plugin_full_width_page object %}
  </div>
</div>
{% endblock %}
