<div class="dropdown">
  <button type="button" class="btn btn-sm btn-purple dropdown-toggle" data-bs-toggle="dropdown" aria-haspopup="true" aria-expanded="false">
    <i class="mdi mdi-download"></i>&nbsp;Export
  </button>
  <ul class="dropdown-menu dropdown-menu-end">
    <li><a class="dropdown-item" href="?{% if url_params %}{{ url_params }}&{% endif %}export=table">Current View</a></li>
    <li><a class="dropdown-item" href="?{% if url_params %}{{ url_params }}&{% endif %}export">All Data ({{ data_format }})</a></li>
    {% if export_templates %}
      <li>
        <hr class="dropdown-divider">
      </li>
      {% for et in export_templates %}
        <li>
          <a class="dropdown-item" href="?{% if url_params %}{{ url_params }}&{% endif %}export={{ et.name }}"
            {% if et.description %} title="{{ et.description }}"{% endif %}
          >
            {{ et.name }}
          </a>
        </li>
      {% endfor %}
    {% endif %}
    {% if perms.extras.add_exporttemplate %}
      <li>
        <hr class="dropdown-divider">
      </li>
      <li>
        <a class="dropdown-item" href="{% url 'extras:exporttemplate_add' %}?content_type={{ content_type.pk }}">Add export template...</a>
      </li>
    {% endif %}
  </ul>
</div>
