{% extends 'base/layout.html' %}
{% load helpers %}
{% load form_helpers %}

{% comment %}
Context:
  object: Python instance of the object being deleted
  form: The delete confirmation form
  form_url: URL for form submission (optional; defaults to current path)
  return_url: The URL to which the user is redirected after submitting the form
{% endcomment %}

{% block title %}Delete {{ object|meta:"verbose_name" }}?{% endblock %}

{% block header %}{% endblock %}

{% block content %}
  <div class="modal" tabindex="-1" style="display: block; position: static">
    <div class="modal-dialog">
      <div class="modal-content" >
        {% include 'htmx/delete_form.html' %}
      </div>
    </div>
  </div>
{% endblock %}
