{% load form_helpers %}

{% render_field form.status %}
{% render_field form.type %}
{% render_field form.tenant_group %}
{% render_field form.tenant %}
{% render_field form.label %}
{% render_field form.color %}
<div class="row mb-3">
    <label class="col-sm-3 col-form-label text-lg-end">{{ form.length.label }}</label>
    <div class="col-md-5">
        {{ form.length }}
    </div>
    <div class="col-md-4">
        {{ form.length_unit }}
    </div>
    <div class="invalid-feedback"></div>
</div>
{% render_field form.tags %}
{% if form.custom_fields %}
  <div class="field-group">
    <div class="row mb-3">
      <h5 class="offset-sm-3">Custom Fields</h5>
    </div>
    {% render_custom_fields form %}
  </div>
{% endif %}
